unit exVariantList;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Variants,
  exStrings, exPair, exnamedVariant;

type

  { TVariantList }

  TVariantList = class(TComponent, IStringConvable)
    constructor Create(const aOwner: TComponent = nil); reintroduce;
    constructor Create(const aItems: array of variant);
  private
    fList: TList;
    fAutoCreate: boolean;
    procedure Clean;
    procedure Initialize;
    function getItem(const aName: string): variant;
    procedure setItem(const aName: string; const AValue: variant);
    function getTheItem(const aIndex: integer): TNamedVariant;
    procedure setTheItem(const aIndex: integer; const AValue: TNamedVariant);
    function getItemExists(const aName: string): boolean;
    function getCount: integer;
  public
      {List contains TNamedVariants}
    property thelist: TList read fList;

      {Count of elements}
    property Count: integer read getCount;

      {Typecasted TNamedVariants}
    property list[const aIndex: integer]: TNamedVariant
      read getTheItem write setTheItem;

      {Find item by name}
    function find(const aName: string): integer;

      {Check if item exists}
    property ItemExists[const aName: string]: boolean read getItemExists;

      {Items themselves, default property}
    property Items[const aName: string]: variant
      read getItem write setItem; default;

      {A new item}
    function Add(const aName: string; const aValue: variant): boolean;

    function Add(const a: array of Variant): boolean;

      {AutoCreate items on :=ing}
    property AutoCreate: boolean read fAutoCreate write fAutoCreate;

      {List as text, similar to TGenericList.asText}
    function asText(const aOpts: TListTextOptions = []): string;

      {Get list as text}
    function getAsString: string;

      {Delete specified by name item}
    function Delete(const aName: string): integer;

      {Delete all items}
    procedure DeleteAll;

      {Destructor}
    destructor Destroy; override;
  end;

operator := (const aObject: TObject): variant;

operator := (const aVar: variant): TObject;

implementation

operator := (const aObject: TObject): variant;
begin
  result:=unassigned;
  result:=ptrUInt(aObject);
end;

operator := (const aVar: variant): TObject;
begin
  result:=TObject(ptrUInt(aVar));
end;


{ TVariantList }

constructor TVariantList.Create(const aOwner: TComponent);
begin
  clean;
  Initialize;
end;

constructor TVariantList.Create(const aItems: array of variant);
begin
  clean;
  initialize;
  add(aItems);
end;

procedure TVariantList.Clean;
begin
  fList:=nil;
  AutoCreate:=true;
end;

procedure TVariantList.Initialize;
begin
  fList:=TList.Create;
end;

function TVariantList.getItem(const aName: string): variant;
begin
  if not ItemExists[aName] then exit(null);
  result:=list[ find(aName) ].Value;
end;

procedure TVariantList.setItem(const aName: string; const AValue: variant);
begin
  case ItemExists[aName] of
    true: list[ find(aName) ].Value:=aValue;
    false: if AutoCreate then Add(aName, aValue);
  end;
end;

function TVariantList.getTheItem(const aIndex: integer): TNamedVariant;
begin
  result:=nil;
  if self = nil then exit;
  if thelist = nil then exit;
  result:=TNamedVariant(thelist[aIndex]);
end;

procedure TVariantList.setTheItem(const aIndex: integer;
  const AValue: TNamedVariant);
begin
  if self = nil then exit;
  if thelist = nil then exit;
  thelist[aIndex]:=aValue;
end;

function TVariantList.getItemExists(const aName: string): boolean;
begin
  result:=find(aName) >= 0;
end;

function TVariantList.getCount: integer;
begin
  if (self = nil) then exit(-1);
  if (thelist = nil) then exit(0);
  result:=thelist.Count;
end;

function TVariantList.find(const aName: string): integer;
var
  i: integer;
begin
  result:=-1;
  if self = nil then exit;
  for i:=0 to Count - 1 do
    if list[i].Name = aName then exit(i);
end;

function TVariantList.Add(const aName: string; const aValue: variant): boolean;
begin
  result:=false;
  if ItemExists[aName] then exit(false);
  thelist.Add( TNamedVariant.Create(aName, aValue) );
  result:=true;
end;

function TVariantList.Add(const a: array of Variant): boolean;
var
  i: integer;
begin
  result:=false;
  if self = nil then exit;
  if length(a) mod 2 = 1 then exit;
  for i:=0 to length(a) - 1 do
  begin
    if i mod 2 = 1 then continue;
    Add(AsString(a[i]), a[i + 1]);
  end;
  result:=true;
end;

function TVariantList.asText(const aOpts: TListTextOptions): string;
const
  _s_nilVL = 'Nil VariantList';
  _s_emptyVL = 'Empty VariantList';
var
  i: integer;
begin
  if self = nil then exit(_s_nilVL);
  if count = 0 then exit(_s_emptyVL);
  result:='';
  for i:=0 to count - 1 do
  begin
    if (tlNumbers in aOpts) then
    begin
      result:=result + asString(i);
      if tlBRs in aOpts then result:=result + ': ' else result:=result + ' = ';
    end;
    if (tlNames in aOpts) then
    begin
      result:=result + list[i].Name;
      if tlBRs in aOpts then result:=result + ': ' else result:=result + ' = ';
    end;
    if tlQuotes in aOpts then result:=result + '"';
    result:=result + asString([ list[i].Value ]);
    if tlQuotes in aOpts then result:=result + '"';
    case tlBRs in aOpts of
      true: result:=result + LineEnding;
      false: if (i <> count - 1) then result:=result + ', ';
    end;
  end;
  if tlHead in aOpts then
  begin
    if tlBRs in aOpts then result:=LineEnding + result;
    result:=asString(self, true) + '(' + asString(self.count) + ') ' + result;
  end;
end;

function TVariantList.getAsString: string;
begin
  result:=asText([tlHead]);
end;

function TVariantList.Delete(const aName: string): integer;
begin
  result:=find(aName);
  if result = -1 then exit;
  list[result].value:=unassigned;
  thelist.Delete(result);
end;

procedure TVariantList.DeleteAll;
var
  i: integer;
begin
  if self = nil then exit;
  for i:=0 to Count - 1 do begin
    list[i].Value:=unassigned;
    list[i].Free;
  end;
  thelist.Clear;
end;

destructor TVariantList.Destroy;
begin
  DeleteAll;
  freeAndNil(fList);
  inherited Destroy;
end;

end.

